package com.persagy.fm.sop.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.persagy.fm.common.exception.BusinessValidatorException;
import com.persagy.fm.sop.mapper.SopMapper;
import com.persagy.fm.sop.model.dto.*;
import com.persagy.fm.sop.model.vo.*;
import com.persagy.fm.sop.service.*;
import com.persagy.old.common.CommonConst;
import com.persagy.old.common.DateUtil;
import com.persagy.old.common.SopConst;
import com.persagy.old.dao.DBConst;
import com.persagy.old.dao.DBConst.Result;
import com.persagy.old.service.ObjectServiceI;
import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang.BooleanUtils;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.lang.reflect.InvocationTargetException;
import java.util.*;

@Service
public class GroupSopServiceImpl implements GroupSopService {
    //@Autowired
    public ObjectServiceI objectService;

    @Autowired
    private SopService sopService;

    @Autowired
    private SopEditService sopEditService;

    @Autowired
    private SopSopRelService sopSopRelService;

    @Autowired
    private SopReviseMsgService sopReviseMsgService;





    @Override
    @Transactional
    public boolean addPublishedSop(SaveSopVo saveSopVo) throws Exception {

        //找到sop 与与对象、设备型号、工单类型、自定义标签、sop关系 并插入数据库中
        addSopRelation(saveSopVo);
        //落库本条sop*/
        //填充必要字段

        //将 请求体中的参数拷贝到我们里面
        //将对象映射到数据库实体上面去
        Sop sop = SopMapper.INSTANCE.SaveSopVoToSop(saveSopVo);
        sop.setSopStatus(SopConst.SOP_STATUS_PUBLISH);
        sop.setPublishStatus(SopConst.PUBLISH_STATUS_NORMAL);
        sop.setVersionCount(1);
        sop.setVersion("V0.1");
        sop.setVersionExplain("[\""+ CommonConst.version_explain_create+"\"]");
        //将参数转化为指定的字段后进行存储
        sop.insert();
        return true;
    }

    @Override
    public boolean verifySopForPublish(SaveSopVo saveSopVo) throws Exception {
        String projectId = saveSopVo.getProject_id();
        List<String> objTypes = Arrays.asList("build","floor","space","system","shaft","equip");

        //验证对象内容是否有效
        //获取所有引用的对象
        List<String> objects = new ArrayList<>();
        List<SaveSopVo.FitObjsBean> fit_objs = saveSopVo.getFit_objs();
        if(CollectionUtils.isNotEmpty(fit_objs)) {
            for (SaveSopVo.FitObjsBean fit_obj : fit_objs) {
                if(objTypes.contains(fit_obj.getObj_type())) {
                    objects.add(fit_obj.getObj_id());
                }
            }
        }
        List<SopStepsBean> steps = saveSopVo.getSteps();
        if(CollectionUtils.isNotEmpty(steps)) {
            for (SopStepsBean step : steps) {
                List<SopStepsBean.StepContentBean> step_content = step.getStep_content();

                if(CollectionUtils.isNotEmpty(step_content)) {
                    for (SopStepsBean.StepContentBean stepContentBean : step_content) {
                        //content_objs
                        List<SopStepsBean.StepContentBean.ContentObjsBean> content_objs = stepContentBean.getContent_objs();
                        if(CollectionUtils.isNotEmpty(content_objs)) {
                            for (SopStepsBean.StepContentBean.ContentObjsBean content_obj : content_objs) {
                                if(objTypes.contains(content_obj.getObj_type())) {
                                    objects.add(content_obj.getObj_id());
                                }
                            }
                        }
                        //confirm_result
                        List<SopStepsBean.StepContentBean.ConfirmResultBean> confirm_result = stepContentBean.getConfirm_result();
                        if(CollectionUtils.isNotEmpty(confirm_result)) {
                            for (SopStepsBean.StepContentBean.ConfirmResultBean confirmResultBean : confirm_result) {
                                if(objTypes.contains(confirmResultBean.getObj_type())) {
                                    objects.add(confirmResultBean.getObj_id());
                                }
                            }
                        }
                    }
                }
            }
        }
        if(CollectionUtils.isNotEmpty(objects)) {
            String objectStr = objectService.queryBatchObj(JSONObject.toJSONString(objects), projectId, false);
            JSONObject objectJson = JSONObject.parseObject(objectStr);
            if(DBConst.Result.FAILURE.equals(objectJson.getString(Result.RESULT))){
                throw new Exception(objectStr);
            }
            Map<String, String> objectIdName = objectService.getObjectIdNameMap(objectStr);
            if(!objectIdName.isEmpty()) {
                Map<Object, Object> valiadtorData = new HashMap<>();
                for (String id : objectIdName.keySet()) {
                    valiadtorData.put(objectIdName.get(id),"添加的对象已作废,请前往编辑!");
                }
                throw new BusinessValidatorException(valiadtorData);
            }
        }
        //验证引用sop是否有效

        Set<String> sopRefSops = this.getSopRefSops(saveSopVo);
        if(CollectionUtils.isNotEmpty(sopRefSops)) {

            QueryWrapper<Sop> queryWrapper = new QueryWrapper<>();
            queryWrapper.in("sop_id",sopRefSops).eq("valid",false);
            //查询他所引用的sop 是否有无效的
            List<Sop> sops = sopService.list(queryWrapper);
            if(CollectionUtils.isNotEmpty(sops)) {

                Map<Object, Object> valiadtorData = new HashMap<>();
                for (Sop sop : sops) {
                    valiadtorData.put(sop.getSopName(),"引用的标准操作流程已被作废,请前往编辑!");
                }
                throw new BusinessValidatorException(valiadtorData);
            }
        }
        return true;
    }


    private boolean addSopRelation(SaveSopVo saveSopVo) throws Exception{

        //添加sop与对象关系-sop_obj_rel
        List<SaveSopVo.FitObjsBean> sopRelObjs = getSopRelObjs(saveSopVo);
        if(CollectionUtils.isNotEmpty(sopRelObjs)) {
            for (SaveSopVo.FitObjsBean sopRelObj : sopRelObjs) {

                SopObjRel sopObjRel = new SopObjRel();
                sopObjRel.setSopId(saveSopVo.getSop_id());
                sopObjRel.setObjId(sopRelObj.getObj_id());
                sopObjRel.setObjType(sopRelObj.getObj_type());
                sopObjRel.setValid(true);
                sopObjRel.insert();
            }
        }

        //添加sop与设备型号关系-sop_equip_model_rel
        List<String> equip_model_ids = saveSopVo.getEquip_model_ids();
        distinctCollection(equip_model_ids);
        if(CollectionUtils.isNotEmpty(equip_model_ids)) {
            for (String equip_model_id : equip_model_ids) {
                SopEquipModelRel sopEquipModelRel = new SopEquipModelRel();
                sopEquipModelRel.setEquipModelId(equip_model_id);
                sopEquipModelRel.setSopId(saveSopVo.getSop_id());
                sopEquipModelRel.setValid(true);
                sopEquipModelRel.insert();
            }
        }

        //添加sop与工单类型关系-sop_order_type_rel
        List<String> order_types = saveSopVo.getOrder_type();
        distinctCollection(order_types);
        if(CollectionUtils.isNotEmpty(order_types)) {
            for (String order_type : order_types) {
                SopOrderTypeRel sopOrderTypeRel = new SopOrderTypeRel();
                sopOrderTypeRel.setOrderType(order_type);
                sopOrderTypeRel.setProjectId(saveSopVo.getProject_id());
                sopOrderTypeRel.setValid(true);
                sopOrderTypeRel.setSopId(saveSopVo.getSop_id());
                sopOrderTypeRel.insert();
            }
        }
        //添加sop与自定义标签关系-sop_label_rel
        List<String> labels = saveSopVo.getLabels();
        distinctCollection(labels);
        if(CollectionUtils.isNotEmpty(labels)) {
            for (String label : labels) {
                SopLabelRel sopLabelRel = new SopLabelRel();
                sopLabelRel.setLabel(label);
                sopLabelRel.setSopId(saveSopVo.getSop_id());
                sopLabelRel.setValid(true);
                sopLabelRel.insert();
            }
        }
        //添加sop与sop关系-sop_sop_rel
        Set<String> sopRefSops = getSopRefSops(saveSopVo);
        if(CollectionUtils.isNotEmpty(sopRefSops)) {
            for (String sopRefSop : sopRefSops) {
                SopSopRel sopSopRel = new SopSopRel();
                sopSopRel.setProjectId(saveSopVo.getProject_id());
                sopSopRel.setRelSopId(sopRefSop);
                sopSopRel.setSopId(saveSopVo.getSop_id());
                sopSopRel.setValid(true);
            }
        }

        return true;

    }

    private void distinctCollection(List<String> list) {
        HashSet set = new HashSet(list);
        list.clear();
        list.addAll(set);
    }

    private  List<SaveSopVo.FitObjsBean> getSopRelObjs(SaveSopVo sop) throws InvocationTargetException, IllegalAccessException {
        List<SaveSopVo.FitObjsBean> sopRelObjs = new ArrayList<>();
        Map<String,SaveSopVo.FitObjsBean> objectId_Object = new HashMap<>();

        if(sop != null) {
            //查询适用的对象
            List<SaveSopVo.FitObjsBean> fit_objs = sop.getFit_objs();
            if(CollectionUtils.isNotEmpty(fit_objs)) {
                for (SaveSopVo.FitObjsBean fit_obj : fit_objs) {
                    objectId_Object.put(fit_obj.getObj_id(),fit_obj);
                }
            }
            List<SopStepsBean> steps = sop.getSteps();
            if(CollectionUtils.isNotEmpty(steps)) {
                for (SopStepsBean step : steps) {
                    //如果不是来自引用的对象
                    if(BooleanUtils.isFalse(step.isFrom_sop())) {
                        List<SopStepsBean.StepContentBean> step_content = step.getStep_content();
                        if(CollectionUtils.isNotEmpty(step_content)) {
                            for (SopStepsBean.StepContentBean stepContentBean : step_content) {
                                List<SopStepsBean.StepContentBean.ContentObjsBean> content_objs = stepContentBean.getContent_objs();
                                if(CollectionUtils.isNotEmpty(content_objs)) {
                                    for (SopStepsBean.StepContentBean.ContentObjsBean content_obj : content_objs) {
                                        //进行数值拷贝到一个对象中去
                                        SaveSopVo.FitObjsBean fitObjsBean = new SaveSopVo.FitObjsBean();
                                        BeanUtils.copyProperties(fitObjsBean,content_obj);
                                        objectId_Object.put(content_obj.getObj_id(),fitObjsBean);
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }

        //sop中 适用对象和步骤中的对象
        for (SaveSopVo.FitObjsBean value : objectId_Object.values()) {
            sopRelObjs.add(value);
        }
        return sopRelObjs;
    }

    public Set<String> getSopRefSops(SaveSopVo sop){
        LinkedHashSet<String> sopIds = new LinkedHashSet<>();
        if(sop != null) {

            List<SopStepsBean> steps = sop.getSteps();
            if(CollectionUtils.isNotEmpty(steps)) {
                for (SopStepsBean step : steps) {
                    if (BooleanUtils.isTrue(step.isFrom_sop())) {
                        sopIds.add(step.getSop_id());
                    } else {
                        List<SopStepsBean.StepContentBean> step_content = step.getStep_content();
                        if(CollectionUtils.isNotEmpty(step_content)) {
                            for (SopStepsBean.StepContentBean stepContentBean : step_content) {
                                if (BooleanUtils.isTrue(stepContentBean.isFrom_sop())) {
                                    sopIds.add(stepContentBean.getSop_id());
                                }

                            }
                        }
                    }
                }
            }
        }
        return sopIds;
    }

    //改造该方法使用业务上的数据

    @Override
    public String addDraftSop(SaveSopEditVo saveSopEditVo) throws Exception {
        Set<String> sopRefSops = this.getSopRefSops(saveSopEditVo);
        saveSopEditVo.setSop_id(UUID.randomUUID().toString().replaceAll("-",""));

        //将表单提交的值拷贝过来
        SopEdit sopEdit = SopMapper.INSTANCE.SaveSopEditVoToSopEdit(saveSopEditVo);
        sopEdit.setRefSopIds(new ArrayList<>(sopRefSops));
        insertRecord(sopEdit);
        return sopEdit.getSopId();
    }

    public boolean insertRecord(SopEdit sopEdit) throws Exception {
        if (StringUtils.isBlank(sopEdit.getSopId())) {
            sopEdit.setSopId(UUID.randomUUID().toString().replaceAll("-", ""));
        }
        //sop类型 1 通用    2 项目专用
        sopEdit.setSopType("1");

        if(StringUtils.isBlank(sopEdit.getSopStatus())) {
            //sop状态 0 草稿  1 已发布   2 已作废
            sopEdit.setSopStatus("0");
        }else {
            sopEdit.setSopStatus("1");
        }
        return sopEdit.insert();
    }

    @Override
    public boolean saveDraftSop(SaveDraftSopVo saveSopVo) throws Exception {
        String saveSopVoString = JSONObject.toJSONString(saveSopVo);
        saveSopVo.setProject_id(CommonConst.group_sop_id);
        String user_id = saveSopVo.getUser_id();
        if(StringUtils.isNotBlank(saveSopVo.getSop_id())) {
            //查询sop 待修订消息表
            //根据状态和 sop id 进行查询
            QueryWrapper<SopReviseMsg> queryWrapper = new  QueryWrapper<>();
            queryWrapper.eq("sop_id",saveSopVo.getSop_id()).eq("status","1");
            List<SopReviseMsg> list = sopReviseMsgService.list(queryWrapper);
            if(CollectionUtils.isNotEmpty(list)) {
                for (SopReviseMsg sopReviseMsg : list) {
                    String objId = sopReviseMsg.getObjId();
                    //如果不包含这个对象id
                    if(!saveSopVoString.contains(objId)) {
                        //设置为忽略状态
                        //根据msg_id 和sopid 为条件进行更新
                        sopReviseMsg.setStatus("2");
                        sopReviseMsg.updateById();
                    }
                }
            }

            //根据sopid 和 有效性查询编辑表
            QueryWrapper<SopEdit> queryBySopIdAndVaild = new QueryWrapper<>();
            queryBySopIdAndVaild.eq("sop_id",saveSopVo.getSop_id()).eq("valid",true);
            SopEdit sopEdit = sopEditService.getOne(queryBySopIdAndVaild);
            //数据库对象转化为业务对象
            SaveSopEditVo saveSopEditVo = SopMapper.INSTANCE.SopEditToSaveSopEditVo(sopEdit);
            //查询该改编辑表中引用的对象
            Set<String> sopRefSops = this.getSopRefSops(saveSopEditVo);


            SopEdit update = SopMapper.INSTANCE.SaveSopEditVoToSopEdit(saveSopVo);



            //将引用sop设置为sopRefSops
            update.setRefSopIds(new ArrayList<>(sopRefSops));
            if("1".equals(saveSopVo.getSop_status())) {
                //将状态更新为3  publish_status
                update.setPublishStatus("3");
            }

            if(sopEdit != null) {
                //更新sop  更新条件为sop_id  valid
                QueryWrapper<SopEdit> sopEditQueryWrapper = new QueryWrapper<>();
                sopEditQueryWrapper.eq("sop_id",sopEdit.getSopId()).eq("valid",true);
                sopEditService.update(update,sopEditQueryWrapper);

            }else {
                //插入编辑状态sop
                this.insertRecord(sopEdit);
            }
        }else {

            SopEdit data = SopMapper.INSTANCE.SaveSopEditVoToSopEdit(saveSopVo);
            //插入编辑状态sop
            this.insertRecord(data);
        }
        //根据sopid 更新sop的状态为3
        Sop sop = new Sop();
        sop.setSopId(saveSopVo.getSop_id());
        sop.setPublishStatus("3");
        return sop.updateById();
    }

    @Override
    public boolean verifySopBeforePublish(VerifySopNameVo verifySopNameVo) throws Exception {
        //查询sop 表中是否存在CommonConst.group_sop_id  sop_name  sopId 的数据
        QueryWrapper<Sop> sopQueryWrapper = new QueryWrapper<>();
        sopQueryWrapper.eq("project_id",CommonConst.group_sop_id).eq("sop_name",verifySopNameVo.getSop_name()).ne("sop_id",verifySopNameVo.getSop_id());
        List<Sop> hashSop = sopService.list(sopQueryWrapper);
        //存在名称
        if(CollectionUtils.isNotEmpty(hashSop)) {
            return false;
        }
        QueryWrapper<SopEdit> sopEditQueryWrapper = new QueryWrapper<>();
        sopEditQueryWrapper.eq("project_id",CommonConst.group_sop_id).eq("sop_name",verifySopNameVo.getSop_name()).ne("sop_id",verifySopNameVo.getSop_id());
        List<SopEdit> hasSopEdit = sopEditService.list(sopEditQueryWrapper);
        //存在名称
        if(CollectionUtils.isNotEmpty(hasSopEdit)) {
            return false;
        }
        return true;
    }



    @Override
    public List<RefSopInfo> queryReferencedSopListById(QuerySopReferenceVo querySopReferenceVo) throws Exception {
        //根据id  和版本进行数据的查询
        QueryWrapper<Sop> sopQueryWrapper = new QueryWrapper<>();
        sopQueryWrapper.eq("sop_id",querySopReferenceVo.getSop_id()).eq("version",querySopReferenceVo.getVersion());
        Sop one = sopService.getOne(sopQueryWrapper);
        // 1通过sop_sop_rel关系表查询到所有引用当前sop的sopID

        QueryWrapper<SopSopRel> sopSopRelQueryWrapper = new QueryWrapper<>();
        sopSopRelQueryWrapper.in("rel_sop_id",one.getSopId()).eq("valid",true);
        List<SopSopRel> sopSopRels = sopSopRelService.list(sopSopRelQueryWrapper);
        Set<String> ids = new HashSet<>();
        for (SopSopRel sopSopRel : sopSopRels) {
            ids.add(sopSopRel.getRelSopId());
        }
        // 2根据sopID 和有效状态 批量查询sop：name、status、update_time、version、stauts_explain

        QueryWrapper<Sop> sopQueryWrapper2 = new QueryWrapper<>();
        sopQueryWrapper2.in("sop_id",ids).eq("valid",true);
        List<Sop> list = sopService.list(sopQueryWrapper2);

        List<RefSopInfo> refSopInfos = new ArrayList<>();
        if(CollectionUtils.isNotEmpty(list)) {
            for (Sop sop : list) {
                RefSopInfo refSopInfo = new RefSopInfo();
                refSopInfo.setRef_sop_id(sop.getSopId());
                refSopInfo.setRef_sop_name(sop.getSopName());
                refSopInfo.setVersion(sop.getVersion());
                refSopInfo.setSop_type(sop.getSopType());
                refSopInfo.setStauts_explain(getSopStatus(sop));
                /*
                 ********************************
                 * ********************************
                 * ********************************
                 * ********************************
                 * ********************************
                 * ********************************
                 * if (!CommonConst.group_sop_id.equals(sopItem.getString("project_id"))) {
                        	sop.put("project_id", sopItem.getString("project_id"));
                        	sop.put("project_name", projectCache.getProjectNameById(sopItem.getString("project_id")));
						}
                 */

                if(sop.getOperator() != null) {
                    refSopInfo.setOperator(JSONObject.toJSONString(sop.getOperator()));
                }
                refSopInfo.setUpdate_time(DateUtil.transferDateFormat(sop.getUpdateTime(), CommonConst.date_format_save, CommonConst.date_format_show));
                refSopInfos.add(refSopInfo);
            }
        }

        //sop 为历史版本的sop
        /*
        ****************************************************************
        * ********************************
        * ********************************
        * ********************************
        * ********************************
        * ****************************************************************
        * ********************************
        * ********************************
        * ********************************
        * ********************************
        *
        *
        *
         */

        return refSopInfos;
    }


    /**
     * 转换sop状态为汉字
     * @param sop
     * @return
     */
    public String getSopStatus(Sop sop){
        String status = sop.getSopStatus();
        if(SopConst.SOP_STATUS_PUBLISH.equals(status)){
            String publishStatus = sop.getPublishStatus();
            status = "已发布";
            if(SopConst.PUBLISH_STATUS_REVISE.equals(publishStatus)){
                status = "已发布、待修订";
            }else if(SopConst.PUBLISH_STATUS_REVISEING.equals(publishStatus)){
                status = "已发布、编辑中";
            }
        }else{
            status = "已作废";
        }
        return status;
    }
}
